@@ -70,7 +70,7 @@ int security_get_boolean_names(char ***names, int *len)
 				     * (namelist[i]->d_reclen + 1));
 		if (!n[i]) {
 			rc = -1;
-			goto bad;
+			goto bad_freen;
 		}
 		strncpy(n[i], namelist[i]->d_name, namelist[i]->d_reclen + 1);
 	}
@@ -82,14 +82,16 @@ out:
 	}
 	free(namelist);
 	return rc;
-bad:
+bad_freen:
 	for (i = 0; i < *len; i++) {
 		if (n[i])
 			free(n[i]);
 	}
 	free(n);
+bad:
 	goto out;
 }
+hidden_def(security_get_boolean_names)
 
 #define STRBUF_SIZE 3 
 static int get_bool_value(const char *name, char **buf)
@@ -164,6 +166,7 @@ int security_get_boolean_active(const char *name)
 	free(buf);
 	return val;
 }
+hidden_def(security_get_boolean_active)
 
 int security_set_boolean(const char *name, int value)
 {
