@@ -4,7 +4,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include "policy.h"
 #include <stdio.h>
 #include <limits.h>
@@ -29,3 +29,5 @@ int security_disable(void)
 
 	return 0;
 }
+hidden_def(security_disable)
+
