@@ -2,6 +2,7 @@
 S_(BINPOLICY, "/policy/policy")
 S_(CONTEXTS_DIR, "/contexts")
 S_(FILE_CONTEXTS, "/contexts/files/file_contexts")
+S_(HOMEDIR_CONTEXTS, "/contexts/files/homedir_template")
 S_(DEFAULT_CONTEXTS, "/contexts/default_contexts")
 S_(USER_CONTEXTS, "/contexts/users/")
 S_(FAILSAFE_CONTEXT, "/contexts/failsafe_context")
@@ -11,3 +12,5 @@ S_(MEDIA_CONTEXTS, "/contexts/files/media")
 S_(REMOVABLE_CONTEXT, "/contexts/removable_context")
 S_(CUSTOMIZABLE_TYPES, "/contexts/customizable_types")
 S_(USERS_DIR, "/users/")
+S_(SEUSERS, "/seusers")
+S_(TRANSLATIONS, "/setrans.conf")
