@@ -1,5 +1,5 @@
 #include <unistd.h>
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include <stdlib.h>
 #include <errno.h>
 
@@ -15,3 +15,4 @@ void freeconary(security_context_t *con)
 	}
 	free(con);
 }
+hidden_def(freeconary)
