@@ -47,6 +47,50 @@ out:
     freeconary(conary);
     return rc;
 }
+hidden_def(get_default_context_with_role)
+
+int get_default_context_with_rolelevel(const char* user, 
+				       const char *role,
+				       const char *level,
+				       security_context_t fromcon,
+				       security_context_t *newcon) 
+{
+
+    int rc=0;
+    int freefrom = 0;
+    context_t con;
+    char *newfromcon;
+    if (!level) 
+	    return get_default_context_with_role(user, role, fromcon, newcon);
+	
+    if (!fromcon) {
+	    rc = getcon(&fromcon);
+	    if (rc < 0)
+		    return rc;
+	    freefrom = 1;
+    }
+    
+    rc = -1;
+    con=context_new(fromcon);
+    if (!con)
+	    goto out;
+    
+    if (context_range_set(con, level))
+	    goto out;
+    
+    newfromcon = context_str(con);
+    if (!newfromcon)
+	    goto out;
+    
+    rc = get_default_context_with_role(user, role, newfromcon, newcon);
+    
+out:
+    context_free(con);
+    if (freefrom) 
+	    freecon(fromcon);
+    return rc;
+
+}
 
 int get_default_context(const char* user, 
 			security_context_t fromcon,
@@ -60,9 +104,9 @@ int get_default_context(const char* user,
 	    return -1;
 
     *newcon = strdup(conary[0]);
+    freeconary(conary);
     if (!(*newcon))
 	    return -1;
-    freeconary(conary);
     return 0;
 }
 
@@ -288,6 +332,67 @@ static int order_compare(const void *A, const void *B)
 	return strcmp(c1->con, c2->con);
 }
 
+int get_ordered_context_list_with_level (const char *user, 
+					 const char *level, 
+					 security_context_t fromcon, 
+					 security_context_t **list)
+{
+    int rc;
+    int freefrom = 0;
+    context_t con;
+    char *newfromcon;
+
+    if (!level) 
+	    return get_ordered_context_list (user, fromcon, list);
+
+    if (!fromcon) {
+	rc = getcon(&fromcon);
+	if (rc < 0)
+		return rc;
+	freefrom = 1;
+    }
+
+    rc = -1;
+    con=context_new(fromcon);
+    if (!con)
+	    goto out;
+
+    if (context_range_set(con, level))
+	    goto out;
+
+    newfromcon = context_str(con);
+    if (!newfromcon)
+	    goto out;
+
+    rc = get_ordered_context_list (user, newfromcon, list);
+
+out:
+    context_free(con);
+    if (freefrom) 
+	    freecon(fromcon);
+    return rc;
+}
+hidden_def(get_ordered_context_list_with_level)
+
+int get_default_context_with_level(const char *user, 
+				   const char *level,
+				   security_context_t fromcon,
+				   security_context_t *newcon)
+{
+    security_context_t *conary;
+    int rc;
+
+    rc = get_ordered_context_list_with_level(user, level, fromcon, &conary);
+    if (rc <= 0)
+	    return -1;
+
+    *newcon = strdup(conary[0]);
+    freeconary(conary);
+    if (!(*newcon))
+	    return -1;
+    return 0;
+}
+
 int get_ordered_context_list (const char *user, 
 			      security_context_t fromcon, 
 			      security_context_t **list)
@@ -410,6 +515,7 @@ failsafe:
     rc = get_failsafe_context(user, &reachable[0]);
     if (rc < 0) {
 	    freeconary(reachable);
+	    reachable = NULL;
 	    goto out;
     }
     rc = 1; /* one context in the list */
