@@ -12,6 +12,7 @@
 #include <regex.h>
 #include <stdarg.h>
 #include "policy.h"
+#include "context_internal.h"
 
 static void 
 #ifdef __GNUC__
@@ -25,6 +26,41 @@ default_printf(const char *fmt, ...)
 	va_end(ap);
 }
 
+/* If MLS is disabled, strip any MLS level field from the context.
+   This allows file_contexts with MLS levels to be processed on
+   a non-MLS system that otherwise has the same policy. */
+static inline int STRIP_LEVEL(char **context, int mls_enabled)
+{
+	char *str;
+	context_t con;
+	int rc = -1;
+
+	if (mls_enabled)
+		return 0;
+
+	con = context_new(*context);
+	if (!con)
+		return rc;
+
+	if (context_range_set(con,NULL))
+		goto out;
+
+	str = context_str(con);
+	if (!str)
+		goto out;
+
+	str = strdup(str);
+	if (!str)
+		goto out;
+
+	free(*context);
+	*context = str;
+	rc = 0;
+out:
+	context_free(con);
+	return rc;
+}
+
 static void (*myprintf)(const char *fmt, ...) = &default_printf;
 
 void set_matchpathcon_printf(void (*f)(const char *fmt, ...))
@@ -35,23 +71,38 @@ void set_matchpathcon_printf(void (*f)(const char *fmt, ...))
 		myprintf = &default_printf;
 }
 
-static int default_invalidcon(const char *path, unsigned lineno, char *context)
-{
-	if (security_check_context(context) < 0 && errno != ENOENT) {
-		myprintf("%s:  line %u has invalid context %s\n", path, lineno, context);
-		return 1;
-	}
-	return 0;
-}
-
-static int (*myinvalidcon)(const char *p, unsigned l, char *c) = &default_invalidcon;
+static int (*myinvalidcon)(const char *p, unsigned l, char *c) = NULL;
 
 void set_matchpathcon_invalidcon(int (*f)(const char *p, unsigned l, char *c))
+{
+	myinvalidcon = f;
+}
+
+static int default_canoncon(const char *path, unsigned lineno, char **context)
+{
+	char *tmpcon;
+	if (security_canonicalize_context(*context, &tmpcon) < 0) {
+		if (errno == ENOENT)
+			return 0;
+		if (lineno)
+			myprintf("%s:  line %u has invalid context %s\n", path, lineno, *context);
+		else
+			myprintf("%s:  invalid context %s\n", path, *context);
+		return 1;
+	}
+	free(*context);
+	*context = tmpcon;
+	return 0;
+}
+
+static int (*mycanoncon)(const char *p, unsigned l, char **c) = &default_canoncon;
+
+void set_matchpathcon_canoncon(int (*f)(const char *p, unsigned l, char **c))
 {
 	if (f)
-		myinvalidcon = f;
+		mycanoncon = f;
 	else
-		myinvalidcon = &default_invalidcon;
+		mycanoncon = &default_canoncon;
 }
 
 static unsigned int myflags;
@@ -68,6 +119,7 @@ typedef struct spec {
 	char *regex_str;	/* regular expession string for diagnostic messages */
 	char *type_str;		/* type string for diagnostic messages */
 	char *context;		/* context string */
+	int context_valid;      /* context string has been validated/canonicalized */
 	regex_t regex;		/* compiled regular expression */
 	mode_t mode;		/* mode format value */
 	int matches;		/* number of matching pathnames */
@@ -274,11 +326,13 @@ int matchpathcon_filespec_add(ino_t ino, int specind, const char *file)
 	fl->specind = specind;
 	fl->file = malloc(strlen(file) + 1);
 	if (!fl->file)
-		goto oom;
+		goto oom_freefl;
 	strcpy(fl->file, file);
 	fl->next = prevfl->next;
 	prevfl->next = fl;
 	return fl->specind;
+oom_freefl:
+	free(fl);
 oom:
 	myprintf("%s:  insufficient memory for file label entry for %s\n",
 		 __FUNCTION__, file);
@@ -413,10 +467,11 @@ static void spec_hasMetaChars(struct spec *spec)
 	}
 	return;
 }
-static int process_line( const char *path, char *line_buf, int pass, unsigned lineno) {
+static int process_line( const char *path, const char *prefix, char *line_buf, int pass, unsigned lineno, int mls_enabled) {
 	int items, len, regerr;
 	char *buf_p;
 	char *regex, *type, *context;
+	const char *reg_buf;
 	char *anchored_regex;
 	len = strlen(line_buf);
 	if (line_buf[len - 1] == '\n')
@@ -431,7 +486,7 @@ static int process_line( const char *path, char *line_buf, int pass, unsigned li
 		sscanf(line_buf, "%as %as %as", &regex, &type,
 		       &context);
 	if (items < 2) {
-		myprintf("%s:  line %d is missing fields\n, skipping", path, lineno); 
+		myprintf("%s:  line %d is missing fields, skipping\n", path, lineno); 
 		return 0;
 	} else if (items == 2) {
 		/* The type field is optional. */
@@ -439,10 +494,19 @@ static int process_line( const char *path, char *line_buf, int pass, unsigned li
 		context = type;
 		type = 0;
 	}
-	
+
+	reg_buf = regex;
+	len = get_stem_from_spec(reg_buf);
+	if (len && prefix && strncmp(prefix, regex, len)) {
+		/* Stem of regex does not match requested prefix, discard. */
+		free(regex);
+		free(type);
+		free(context);
+		return 0;
+	}
+
 	if (pass == 1) {
 		/* On the second pass, compile and store the specification in spec. */
-		const char *reg_buf = regex;
 		char *cp;
 		spec_arr[nspec].stem_id = find_stem_from_spec(&reg_buf);
 		spec_arr[nspec].regex_str = regex;
@@ -463,11 +527,22 @@ static int process_line( const char *path, char *line_buf, int pass, unsigned li
 			regcomp(&spec_arr[nspec].regex,
 				anchored_regex,
 				REG_EXTENDED | REG_NOSUB);
-		free(anchored_regex);
-		if (regerr < 0) {
-			myprintf("%s:  line %d has invalid regex %s\n", path, lineno, anchored_regex); 
+		if (regerr != 0) {
+			size_t errsz = 0;
+			char *errbuf = NULL;
+			errsz = regerror(regerr, &spec_arr[nspec].regex, 
+					 errbuf, errsz);
+			if (errsz)
+				errbuf = malloc(errsz);
+			if (errbuf)
+				(void) regerror(regerr, 
+						&spec_arr[nspec].regex, 
+						errbuf, errsz);
+			myprintf("%s:  line %d has invalid regex %s:  %s\n", path, lineno, anchored_regex, (errbuf ? errbuf : "out of memory")); 
+			free(anchored_regex);
 			return 0;
 		}
+		free(anchored_regex);
 		
 		/* Convert the type string to a mode format */
 		spec_arr[nspec].type_str = type;
@@ -507,13 +582,42 @@ static int process_line( const char *path, char *line_buf, int pass, unsigned li
 		}
 		
 	skip_type:
-		
-		spec_arr[nspec].context = context;
-		
 		if (strcmp(context, "<<none>>")) {
-			if (myinvalidcon(path, lineno, context))
-				return 0;
+			char *tmpcon = NULL;
+
+			if (myflags & MATCHPATHCON_NOTRANS)
+				goto skip_trans;
+
+			if (context_translations) {
+				if (raw_to_trans_context(context, &tmpcon)) {
+					myprintf("%s: line %u has invalid "
+					         "context %s\n",
+					         path, lineno, context);
+					return 0;
+				}
+				free(context);
+				context = tmpcon;
+			} else {
+				if (STRIP_LEVEL(&context, mls_enabled))
+					return -1;
+			}
+
+skip_trans:
+			if (myflags & MATCHPATHCON_VALIDATE) {
+				if (myinvalidcon) {
+					/* Old-style validation of context. */
+					if (myinvalidcon(path, lineno, context)) 
+						return 0;
+				} else {
+					/* New canonicalization of context. */
+					if (mycanoncon(path, lineno, &context))
+						return 0;
+				}
+				spec_arr[nspec].context_valid = 1;
+			}
 		}
+
+		spec_arr[nspec].context = context;
 		
 		/* Determine if specification has 
 		 * any meta characters in the RE */
@@ -530,7 +634,7 @@ static int process_line( const char *path, char *line_buf, int pass, unsigned li
 	return 0;
 }
 
-int matchpathcon_init(const char *path)
+int matchpathcon_init_prefix(const char *path, const char *prefix)
 {
 	FILE *fp;
 	FILE *localfp = NULL;
@@ -542,6 +646,7 @@ int matchpathcon_init(const char *path)
 	unsigned int lineno, pass, i, j, maxnspec;
 	spec_t *spec_copy=NULL;
 	int status=-1;
+	int mls_enabled=is_selinux_mls_enabled();
 
 	/* Open the specification file. */
 	if (!path)
@@ -575,20 +680,20 @@ int matchpathcon_init(const char *path)
 		lineno = 0;
 		nspec = 0;
 		while (getline(&line_buf, &line_len, fp) > 0 && nspec < maxnspec) {
-			if (process_line(path, line_buf, pass, ++lineno) != 0)
+			if (process_line(path, prefix, line_buf, pass, ++lineno, mls_enabled) != 0)
 				goto finish;
 		}
 		lineno = 0;
 		if (homedirfp) 
 			while (getline(&line_buf, &line_len, homedirfp) > 0 && nspec < maxnspec) {
-				if (process_line(homedir_path, line_buf, pass, ++lineno) != 0)
+				if (process_line(homedir_path, prefix, line_buf, pass, ++lineno, mls_enabled) != 0)
 					goto finish;
 			}
 
 		lineno = 0;
 		if (localfp) 
 			while (getline(&line_buf, &line_len, localfp) > 0 && nspec < maxnspec) {
-				if (process_line(local_path, line_buf, pass, ++lineno) != 0)
+				if (process_line(local_path, prefix, line_buf, pass, ++lineno, mls_enabled) != 0)
 					goto finish;
 			}
 
@@ -635,7 +740,12 @@ int matchpathcon_init(const char *path)
 	if (localfp) fclose(localfp);
 	return status;
 }
-hidden_def(matchpathcon_init)
+hidden_def(matchpathcon_init_prefix)
+
+int matchpathcon_init(const char *path)
+{
+	return matchpathcon_init_prefix(path, NULL);
+}
 
 static int matchpathcon_common(const char *name, 
 			       mode_t mode)
@@ -644,7 +754,7 @@ static int matchpathcon_common(const char *name,
 	const char *buf = name;
 
 	if (!nspec) {
-		ret = matchpathcon_init(NULL);
+		ret = matchpathcon_init_prefix(NULL, NULL);
 		if (ret < 0)
 			return ret;
 		if (!nspec) {
@@ -710,11 +820,28 @@ int matchpathcon(const char *name,
 		return -1;
 	}
 
+	if (!spec_arr[i].context_valid) {
+		if (myinvalidcon) {
+			/* Old-style validation of context. */
+			if (myinvalidcon("file_contexts", 0, spec_arr[i].context))
+				goto bad;
+		} else {
+			/* New canonicalization of context. */
+			if (mycanoncon("file_contexts", 0, &spec_arr[i].context))
+				goto bad;
+		}
+		spec_arr[i].context_valid = 1;
+	}
+
 	*con = strdup(spec_arr[i].context);
 	if (!(*con))
 		return -1;
 
 	return 0;
+
+bad:
+	errno = EINVAL;
+	return -1;
 }
 
 int matchpathcon_index(const char *name, 
