@@ -11,7 +11,7 @@ int rpm_execcon(unsigned int verified __attribute__((unused)),
 {
 	security_context_t mycon = NULL, fcon = NULL, newcon = NULL;
 	context_t con = NULL;
-	int rc;
+	int rc = 0;
 
 	if (is_selinux_enabled() < 1)
 		return execve(filename, argv, envp);
@@ -30,6 +30,7 @@ int rpm_execcon(unsigned int verified __attribute__((unused)),
 
 	if (!strcmp(mycon, newcon)) {
 		/* No default transition, use rpm_script_t for now. */
+		rc = -1;
 		con = context_new(mycon);
 		if (!con)
 			goto out;
@@ -39,13 +40,17 @@ int rpm_execcon(unsigned int verified __attribute__((unused)),
 		newcon = strdup(context_str(con));
 		if (!newcon)
 			goto out;
+		rc = 0;
 	}
 
 	rc = setexeccon(newcon);
 	if (rc < 0) 
 		goto out;
-	rc = execve(filename, argv, envp);
 out:
+
+	if (rc >= 0 || !security_getenforce())
+		rc = execve(filename, argv, envp); 
+
 	context_free(con);
 	freecon(newcon);
 	freecon(fcon);
