@@ -14,21 +14,26 @@
 #define SELINUXDEFAULT "targeted"
 #define SELINUXTYPETAG "SELINUXTYPE="
 #define SELINUXTAG "SELINUX="
+#define SETLOCALDEFS "SETLOCALDEFS="
+#define REQUIRESEUSERS "REQUIRESEUSERS="
 
 /* Indices for file paths arrays. */
 #define BINPOLICY         0
 #define CONTEXTS_DIR      1    
 #define FILE_CONTEXTS     2
-#define DEFAULT_CONTEXTS  3
-#define USER_CONTEXTS     4
-#define FAILSAFE_CONTEXT  5
-#define DEFAULT_TYPE      6
-#define BOOLEANS          7
-#define MEDIA_CONTEXTS    8
-#define REMOVABLE_CONTEXT 9
-#define CUSTOMIZABLE_TYPES    10
-#define USERS_DIR         11
-#define NEL               12
+#define HOMEDIR_CONTEXTS  3
+#define DEFAULT_CONTEXTS  4
+#define USER_CONTEXTS     5
+#define FAILSAFE_CONTEXT  6
+#define DEFAULT_TYPE      7
+#define BOOLEANS          8
+#define MEDIA_CONTEXTS    9
+#define REMOVABLE_CONTEXT 10
+#define CUSTOMIZABLE_TYPES    11
+#define USERS_DIR         12
+#define SEUSERS           13
+#define TRANSLATIONS      14
+#define NEL               15
 
 /* New layout is relative to SELINUXDIR/policytype. */
 static char *file_paths[NEL];
@@ -115,52 +120,96 @@ int selinux_getenforcemode(int *enforce) {
   }
   return ret;
 }
+hidden_def(selinux_getenforcemode)
 
 static char *selinux_policyroot = NULL;
+static char *selinux_rootpath = NULL;
 
-static void init_selinux_policyroot(void) __attribute__ ((constructor));
+static void init_selinux_config(void) __attribute__ ((constructor));
 
-static void init_selinux_policyroot(void)
+static void init_selinux_config(void)
 {
-  char *type=SELINUXDEFAULT;
-  int i=0, len=sizeof(SELINUXTYPETAG)-1, len2;
-  char buf[4097];
-  FILE *cfg;
+  int i, *intptr;
+  size_t rootlen, len;
+  char *line_buf = NULL, *buf_p, *value, *type = NULL, *end;
+  FILE *fp;
+
   if (selinux_policyroot) return;
   if (access(SELINUXDIR, F_OK) != 0) {
 	  selinux_policyroot = SECURITYDIR;
+	  selinux_rootpath = SECURITYDIR;
 	  use_compat_file_path = 1;
 	  return;
   }
-  cfg = fopen(SELINUXCONFIG,"r");
-  if (cfg) {
-    while (fgets_unlocked(buf, 4096, cfg)) {
-      if (strncmp(buf,SELINUXTYPETAG,len)==0) {
-	type=buf+len;
-	break;
-      }
-    }
-    fclose(cfg);
+
+  selinux_rootpath = SELINUXDIR;
+  fp = fopen(SELINUXCONFIG,"r");
+  if (fp) {
+	  while (getline(&line_buf, &len, fp) > 0) {
+		  len = strlen(line_buf); /* reset in case of embedded NUL */
+		  if (line_buf[len - 1] == '\n')
+			  line_buf[len - 1] = 0;
+		  buf_p = line_buf;
+		  while (isspace(*buf_p))
+			  buf_p++;
+		  if (*buf_p == '#' || *buf_p == 0)
+			  continue;
+
+		  if (!strncasecmp(buf_p, SELINUXTYPETAG, 
+				   sizeof(SELINUXTYPETAG)-1)) {
+			  type = strdup(buf_p+sizeof(SELINUXTYPETAG)-1);
+			  if (!type)
+				  return;
+			  end  = type + strlen(type)-1;
+			  while ((end > type) && 
+				 (isspace(*end) || iscntrl(*end))) {
+				  *end = 0;
+				  end--;
+			  }
+			  continue;
+		  } else if (!strncmp(buf_p, SETLOCALDEFS, 
+				      sizeof(SETLOCALDEFS)-1)) {
+			  value = buf_p + sizeof(SETLOCALDEFS)-1;
+			  intptr = &load_setlocaldefs;
+		  } else if (!strncmp(buf_p, REQUIRESEUSERS, 
+				      sizeof(REQUIRESEUSERS)-1)) {
+			  value = buf_p + sizeof(REQUIRESEUSERS)-1;
+			  intptr = &require_seusers;
+		  } else {
+			  continue;
+		  }
+
+		  if (isdigit(*value)) 
+			  *intptr = atoi(value);
+		  else if (strncasecmp(value, "true", sizeof("true")-1))
+			  *intptr = 1;
+		  else if (strncasecmp(value, "false", sizeof("false")-1))
+			  *intptr = 0;
+	  }
+	  free(line_buf);
+	  fclose(fp);
   }
-  i=strlen(type)-1;
-  while ((i>=0) && 
-	 (isspace(type[i]) || iscntrl(type[i]))) {
-    type[i]=0;
-    i--;
+
+  if (!type) {
+	  type = strdup(SELINUXDEFAULT);
+	  if (!type)
+		  return;
   }
-  len=sizeof(SELINUXDIR) + strlen(type);
-  selinux_policyroot=malloc(len);
+  rootlen = sizeof(SELINUXDIR) + strlen(type);
+  selinux_policyroot = malloc(rootlen);
   if (!selinux_policyroot)
 	  return;
-  snprintf(selinux_policyroot,len, "%s%s", SELINUXDIR, type);
-  
+  snprintf(selinux_policyroot, rootlen, "%s%s", 
+	   SELINUXDIR, type);
+  free(type);
+
   for (i = 0; i < NEL; i++) {
-	  len2 = len + strlen(file_path_suffixes_data.str
-			      + file_path_suffixes_idx[i])+1;
-	  file_paths[i] = malloc(len2);
+	  len = rootlen + strlen(file_path_suffixes_data.str
+				 + file_path_suffixes_idx[i])+1;
+	  file_paths[i] = malloc(len);
 	  if (!file_paths[i])
 		  return;
-	  snprintf(file_paths[i], len2, "%s%s", selinux_policyroot,
+	  snprintf(file_paths[i], len, "%s%s", selinux_policyroot,
 		   file_path_suffixes_data.str + file_path_suffixes_idx[i]);
   }
   use_compat_file_path = 0;
@@ -201,6 +250,11 @@ const char *selinux_policy_root() {
 	return selinux_policyroot;
 }
 
+const char *selinux_path() {
+	return selinux_rootpath;
+}
+hidden_def(selinux_path)
+
 const char *selinux_default_context_path() {
   return get_path(DEFAULT_CONTEXTS);
 }
@@ -219,12 +273,18 @@ hidden_def(selinux_removable_context_path)
 const char *selinux_binary_policy_path() {
   return get_path(BINPOLICY);
 }
+hidden_def(selinux_binary_policy_path)
 
 const char *selinux_file_context_path() {
   return get_path(FILE_CONTEXTS);
 }
 hidden_def(selinux_file_context_path)
 
+const char *selinux_homedir_context_path() {
+  return get_path(HOMEDIR_CONTEXTS);
+}
+hidden_def(selinux_homedir_context_path)
+
 const char *selinux_media_context_path() {
   return get_path(MEDIA_CONTEXTS);
 }
@@ -254,3 +314,14 @@ const char *selinux_users_path() {
 }
 hidden_def(selinux_users_path)
 
+const char *selinux_usersconf_path() {
+  return get_path(SEUSERS);
+}
+hidden_def(selinux_usersconf_path)
+
+const char *selinux_translations_path() 
+{
+  return get_path(TRANSLATIONS);
+}
+hidden_def(selinux_translations_path)
+
