@@ -4,7 +4,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
-#include <selinux/selinux.h>
+#include "selinux_internal.h"
 #include "policy.h"
 #include <stdio.h>
 #include <limits.h>
@@ -28,3 +28,4 @@ int security_setenforce(int value)
 
 	return 0;
 }
+hidden_def(security_setenforce)
