@@ -13,7 +13,7 @@ void usage(const char *progname) {
 
 int main(int argc, char **argv) 
 {
-	int rc;
+	int rc=0;
 	if (argc != 2) {
 		usage(argv[0]);
 	}
